@@ -1,22 +1,13 @@
-
-import pyxnat
 import re
-import os
+
 
 SGP_PATTERN = '\w+-x-\w+-x-\w+_v[0-9]+-x-[0-9a-f]+'
 
 
-def __pyxnat_assessor_type():
-    return pyxnat.core.resources.Assessor
+def full_label(project, subject, session, assessor):
+    return '-x-'.join([project, subject, session, assessor])
 
 
-def full_label(project_id, subject_id, session_id,
-               assessor_label):
-    return '-x-'.join([project_id, subject_id, session_id, assessor_label])
-
-
-# TODO: BenM/assessor_of_assessor/behaviour contingent on assessor version;
-# the name shouldn't be used to test this
 def full_label_from_assessor(assessor):
     assessor_label = assessor.label()
     if '-x-' in assessor_label:
@@ -64,6 +55,6 @@ def parse_full_assessor_name(assessor_name):
     return assrdict
 
 
-def is_sgp_assessor(dirpath):
-    # PROJECT-x-SUBJECT-x-PROCTYPE-x-UID
-    return re.match(SGP_PATTERN, os.path.basename(dirpath))
+def is_sgp_assessor(assessor):
+    # Try to match the assessor label with the SGP pattern
+    return re.match(SGP_PATTERN, assessor)
