@@ -3,15 +3,15 @@
 
 """ launcher.py that represents the main object called by the executables """
 
-
 from datetime import datetime, timedelta
 import logging
 import sys
 import os
 import traceback
-import socket
 import tempfile
+import time
 
+from . import lockfiles
 from . import processors, modules, XnatUtils, task, cluster, processors_v3
 from .task import Task, ClusterTask, XnatTask, mkdirp
 from .dax_settings import DAX_Settings, DAX_Netrc
@@ -86,7 +86,10 @@ class Launcher(object):
                  project_modules_dict=dict(),
                  yaml_dict=dict(),
                  priority_project=None,
-                 queue_limit='10',
+                 queue_limit=10,
+                 queue_limit_pending=10,
+                 limit_pendinguploads=1,
+                 launch_delay_sec=1,
                  root_job_dir='/tmp',
                  xnat_user=None, xnat_pass=None, xnat_host=None, cr=None,
                  job_email=None, job_email_options='FAIL', job_rungroup=None,
@@ -102,6 +105,9 @@ class Launcher(object):
         :param project_modules_dict: dictionary associating project & modules
         :param priority_project: list of project to describe the priority
         :param queue_limit: maximum number of jobs in the queue
+        :param queue_limit_pending: maximum pending jobs in the queue
+        :param limit_pendinguploads: maximum number of uploads waiting
+        :param launch_delay_sec: time to wait between job launches
         :param root_job_dir: root directory for jobs
         :param xnat_host: XNAT Host url. By default, use env variable.
         :param cr: True if the host is an XNAT CR instance (will default to
@@ -115,6 +121,9 @@ class Launcher(object):
         :return: None
         """
         self.queue_limit = queue_limit
+        self.queue_limit_pending = queue_limit_pending
+        self.limit_pendinguploads = limit_pendinguploads
+        self.launch_delay_sec = launch_delay_sec
         self.root_job_dir = root_job_dir
         self.resdir = resdir
         self.smtp_host = smtp_host
@@ -274,68 +283,66 @@ name as a key and list of yaml filepaths as values.'
         """
         return assr_info['procstatus'] == task.NEED_TO_RUN
 
-    def launch_tasks(self, task_list, writeonly=False, pbsdir=None,
-                     force_no_qsub=False):
+    def launch_tasks(self, task_list, force_no_qsub=False):
         """
         Launch tasks from the passed list until the queue is full or
          the list is empty
 
         :param task_list: list of task to launch
-        :param writeonly: write the job files without submitting them
-        :param pbsdir: folder to store the pbs file
         :param force_no_qsub: run the job locally on the computer (serial mode)
         :return: None
         """
-
-        # TODO: determine if writeonly still does anything?
-
-        if force_no_qsub:
-            LOGGER.info('No qsub - Running job locally on your computer.')
-        else:
-            # Check number of jobs on cluster
-            cjobs = cluster.count_jobs()
-            if cjobs == -1:
-                LOGGER.error('cannot get count of jobs from cluster')
-                return
-
-            if cluster.command_found(cmd=DAX_SETTINGS.get_cmd_submit()):
-                LOGGER.info('%s jobs currently in queue' % str(cjobs))
+        launched, pending, pendinguploads = cluster.count_jobs(self.resdir,force_no_qsub)
+        if not force_no_qsub:
+            LOGGER.info(
+                'Cluster: %d/%d total, %d/%d pending, %d/%d pending uploads',
+                launched, self.queue_limit,
+                pending, self.queue_limit_pending,
+                pendinguploads, self.limit_pendinguploads
+                )
 
         # Launch until we reach cluster limit or no jobs left to launch
-        while (cjobs < self.queue_limit or writeonly) and len(task_list) > 0:
+        while(
+                launched < self.queue_limit and
+                pending < self.queue_limit_pending and
+                pendinguploads < self.limit_pendinguploads and
+                len(task_list) > 0
+                ):
+
             cur_task = task_list.pop()
 
-            if writeonly:
-                msg = "+Writing PBS file for job:%s, currently %s jobs in \
-cluster queue"
-                LOGGER.info(msg % (cur_task.assessor_label,
-                                   str(cjobs)))
-            else:
-                msg = '+Launching job:%s, currently %s jobs in cluster queue'
-                LOGGER.info(msg % (cur_task.assessor_label, str(cjobs)))
+            LOGGER.info('Launching job: %s', cur_task.assessor_label)
 
             try:
                 success = cur_task.launch(force_no_qsub=force_no_qsub)
             except Exception as E:
-                LOGGER.critical('Caught exception launching job %s'
-                                % cur_task.assessor_label)
-                LOGGER.critical('Exception class %s caught with message %s'
-                                % (E.__class__, str(E)))
+                LOGGER.critical(
+                    'Caught exception launching job %s',
+                    cur_task.assessor_label
+                    )
+                LOGGER.critical(
+                    'Exception class %s caught with message %s',
+                    E.__class__, str(E)
+                    )
                 LOGGER.critical(traceback.format_exc())
-
                 success = False
 
             if not success:
                 LOGGER.error('ERROR: failed to launch job')
                 raise ClusterLaunchException
 
-            cjobs = cluster.count_jobs()
+            time.sleep(self.launch_delay_sec)
+
+            launched, pending, pendinguploads = cluster.count_jobs(self.resdir,force_no_qsub)
+            if not force_no_qsub:
+                LOGGER.info(
+                    'Cluster: %d/%d total, %d/%d pending, %d/%d pending uploads',
+                    launched, self.queue_limit,
+                    pending, self.queue_limit_pending,
+                    pendinguploads, self.limit_pendinguploads
+                    )
 
-            if cjobs == -1:
-                LOGGER.error('ERROR: cannot get count of jobs from cluster')
-                raise ClusterCountJobsException
 
-    # UPDATE Main Method
     def update_tasks(self, lockfile_prefix, project_local, sessions_local):
         """
         Main method to Update the tasks
@@ -450,7 +457,9 @@ cluster queue"
                                        mod_delta=mod_delta, lastrun=lastrun,
                                        start_sess=start_sess)
 
-                    self.build_project_subjgenproc(intf, project_id)
+                    if len(self.get_subjgenproc_processors(project_id)) > 0:
+                        # The project has SGP processors so we build them
+                        self.build_project_subjgenproc(intf, project_id)
 
                 except Exception as E:
                     err1 = 'Caught exception building project %s'
@@ -1035,7 +1044,7 @@ The project is not part of the settings."""
                 LOGGER.error(mess_str)
                 exit(1)
         else:
-            success = self.lock_flagfile(flagfile)
+            success = lockfiles.lock_flagfile(flagfile)
             if not success:
                 LOGGER.warn('failed to get lock. Already running.')
                 exit(1)
@@ -1056,39 +1065,7 @@ The project is not part of the settings."""
         :return: None
         """
         if not project_local:
-            self.unlock_flagfile(flagfile)
-
-    @staticmethod
-    def lock_flagfile(lock_file):
-        """
-        Create the flagfile to lock the process
-
-        :param lock_file: flag file use to lock the process
-        :return: True if the file didn't exist, False otherwise
-        """
-        if os.path.exists(lock_file):
-            return False
-        else:
-            open(lock_file, 'w').close()
-
-            # Write hostname-PID to lock file
-            _pid = os.getpid()
-            _host = socket.gethostname().split('.')[0]
-            with open(lock_file, 'w') as f:
-                f.write('{}-{}'.format(_host, _pid))
-
-            return True
-
-    @staticmethod
-    def unlock_flagfile(lock_file):
-        """
-        Remove the flagfile to unlock the process
-
-        :param lock_file: flag file use to lock the process
-        :return: None
-        """
-        if os.path.exists(lock_file):
-            os.remove(lock_file)
+            lockfiles.unlock_flagfile(flagfile)
 
     def get_tasks(self, xnat, is_valid_assessor, project_list=None,
                   sessions_local=None):
